import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:matrix_flutter_app_v2/animate/fade.dart';
import 'package:video_player/video_player.dart';

import 'package:wakelock/wakelock.dart';

class CustomVideoPlayer extends StatefulWidget {
  final String url;

  const CustomVideoPlayer({super.key, required this.url});

  @override
  State<StatefulWidget> createState() => _CustomVideoPlayerState();
}

class _CustomVideoPlayerState extends State<CustomVideoPlayer> {
  VideoPlayerController? _controller;
  Timer? _timer;
  List<Widget> showWidget = [];

  _CustomVideoPlayerState();

  @override
  void initState() {
    super.initState();
    WidgetsFlutterBinding.ensureInitialized();
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersiveSticky,
        overlays: [SystemUiOverlay.bottom]);
    SystemChrome.setPreferredOrientations(
        [DeviceOrientation.landscapeLeft, DeviceOrientation.landscapeRight]);
    Wakelock.enable();
    initVideoPlayer();
  }

  void initVideoPlayer() {
    _controller = VideoPlayerController.networkUrl(Uri.parse(widget.url))
      ..initialize().then((_) {
        // Ensure the first frame is shown after the video is initialized, even before the play button has been pressed.
        setState(() {
          _controller!.play();
        });
      });
    _controller!.addListener(() {});
    setState(() {
      showWidget.add(
        GestureDetector(
          onTap: (){
            VideoPlayerValue value = _controller!.value;
            if(value.isPlaying) {
              if(value.isCompleted){
                _controller!.seekTo(Duration.zero);
              } else {
                _controller!.pause();
              }
            } else {
              _controller!.play();
            }
          },
          child: VideoPlayer(_controller!),
        ),
      );
    });

    showWidget.add(
      const Fade(
        child: Opacity(
          opacity: 0.8,
          child: Dialog.fullscreen(
            child: Center(
              child: Text("测试"),
            ),
          ),
        ),
      ),
    );

    Timer(const Duration(seconds: 1), () {
      setState(() {
        showWidget.removeLast();
      });
    });
  }

  void onKey(RawKeyEvent event) {
    if (event is RawKeyUpEvent) return;
    RawKeyEventDataAndroid e = event.data as RawKeyEventDataAndroid;
    switch (e.keyCode) {
      case 21:
        int pos = _controller!.value.position.inMilliseconds;
        _controller!.seekTo(Duration(milliseconds: pos - 30000));
        break;
      case 22:
        int pos = _controller!.value.position.inMilliseconds;
        _controller!.seekTo(Duration(milliseconds: pos + 30000));
        break;
      case 62:
        if (_controller!.value.isPlaying) {
          _controller!.pause();
        } else {
          _controller!.play();
        }
        break;
      default:
        debugPrint("$e");
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Colors.black,
        body: PopScope(
          onPopInvoked: (val) {
            if (!val) {
              return;
            }
            _controller!.pause();
          },
          child: Center(
            child: _controller!.value.isInitialized
                ? AspectRatio(
                    aspectRatio: _controller!.value.aspectRatio,
                    child: RawKeyboardListener(
                      onKey: onKey,
                      autofocus: true,
                      focusNode: FocusNode(),
                      child: Stack(
                        children: showWidget,
                      ),
                    ),
                  )
                : Container(),
          ),
        ));
  }

  @override
  void dispose() {
    Wakelock.disable();
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersiveSticky,
        overlays: [SystemUiOverlay.top]);
    SystemChrome.setPreferredOrientations([]);
    _controller?.dispose();
    super.dispose();
  }
}
